% Assemble_Inputs.m
% Jason Wu
% Edited 1/26/15
%
% This function serves to compile the input parameters for the simulation
% of ground motion, calculation of pipe fragilities, and GIRAFFE
% simulations. 
%
% The user is instructed to fill out the variables listed in the section
% named USER DEFINED INPUTS. This section is further subdivided into the
% following sections: 
% -File Organization 
% -Flags/Options
% -Random Field Simulation Options
% -Ground Motion Parameters.
% 
% Description of Inputs:
%
% **File Organization**
% Identifier - String - Name of the current project as will be indicated on
%       the Master Folder
% siteListFilename - String - Filename of the site list provided by the
%       user. The code will look inside the subfolder UserInput for this
%       file. This file must be a txt file (e.g. siteList.txt).
%       If the user wishes for the site list to be generated by the
%       code, set this equal to the empty string ''. If the user is
%       providing the site list, then the first column must be latitude
%       coordinates, and the second column must be longitude coordinates.
%       If the user is providing Vs30 values, then the third column
%       contains the corresponding Vs30 values.
% EPAFilename - String - Filename of the EPANET system definition file.
%       The code will look inside the subfolder UserInput for this file.
%       This file must be a txt or inp file (e.g. network.inp).
% GISFilename - String - Filename of the excel file containing pipe data
%       obtained from GIS. Refer to the file GIS_Data_Template in the
%       UserInput subfolder. The columns required are as follows:
%       OBJECTID - Numeric - Describes an ID number for a particular pipe
%       Xcoordstart - Numeric - Describes the longitudinal coordinate of
%               the start of the pipe. Note that this coordinate must be in
%               the same units as those found in the EPANET system
%               definition file. (Typically meters)
%       Ycoordstart - Numeric - Describes the latitudinal coordinate of the
%               start of the pipe. Note that this coordinate must be in the
%               same units as those found in the EPANET system definition
%               file. (Typically meters)
%       Xcoordend - Numeric - Describes the lognitudinal coordinate of the
%               the end of the pipe. Note that this coordinate must be in
%               the same units as those found in the EPANET system
%               definition file. (Typically meters)
%       Ycoordend - Numeric - Describes the latitudinal coordinate of the
%               end of the pipe. Note that this coordinate must be in the
%               same units as those found in the EPANET system definition
%               file. (Typically meters)
%       XcentDD - Numeric - Describes the longitudinal coordinate of the
%               centroid of the pipe in decimal degrees.
%       YcendDD - Numeric - Describes the latitudinal coordinate of the
%               centroid of the pipe in decimal degrees.
%       Material - String or Numeric - Describes the material (or proxy
%               parameter) of the pipe. 
% GISSheetname - String - Describes the name of the sheet inside the excel
%       file that the data is stored in.
%
% **Flags/Options**
% OverwriteAllFlag - Logical - Enables or disables the overwriting of data. If the user
%       has previously calculated parts of the code, the user may disable
%       overwriting so that the code does not perform new calculations.
%       0 = disabled, 1 = enabled.
% OverwriteOpenSHAFlag - Logical  Enables or disables the overwriting of 
%       data related to OpenSHA calculations. If the user has previously 
%       calculated parts of the code related to the site list creation or 
%       OpenSHA input/output creation, the user may disable overwriting so 
%       that the code does not perform new calculations. 
%       0 = disabled, 1 = enabled. 
%       Note that the flag OverwriteAllFlag must be disabled for this
%       flag to have effect
% OverwriteGMSimFlag - Logical - Enables or disables the overwriting of 
%       data related to ground motion simulation. If the user has previously 
%       calculated parts of the code related to ground motion simulation
%       results, the user may disable overwriting so that the code does not 
%       perform new calculations. 
%       0 = disabled, 1 = enabled. 
%       Note that the flag OverwriteAllFlag must be disabled for this
%       flag to have effect
% SuppressMessagesFlag - Logical - Enables or disables the printing of messages while
%       the code is running. 
%       0 = disabled (messages show)
%       1 = enabled (messages suppressed)
% DefineVs30Flag - Logical - Enables or disables the reading of Vs30 values in the
%       site list file. If the user defines Vs30 values in the site list,
%       this flag must be enabled. If this flag is disabled, the code will
%       allow OpenSHA to estimate Vs30 values using topographic slope.
%       0 = disabled, 1 = enabled
% DefineConstVs30Flag  Logical  Enables or disables the specification of 
%       a constant Vs30 value to all sites in the site list. If this flag 
%       is disabled, the code will determine Vs30 values from a user 
%       specified site list or allow OpenSHA to estimate Vs30 values using 
%       topographic slope. 0 = disabled, 1 = enabled. Alternatively, the 
%       user may specify false or true, respectively. See ConstVs30 to 
%       specify the Vs30 value.
%       Note that the flag DefineVs30Flag must be enabled for this flag to 
%       have effect. If both this flag and DefineVs30Flag are enabled, and 
%       the user specified site list contains Vs30 values, these Vs30 values 
%       will be used instead of the specified constant Vs30 value.
% RandomFieldFlag - Scalar - Selects the methodology to generate correlated
%       epsilons. 
%       1 = covariance formulation
%       2 = spectral formulation
%       3 = sequential formulation
%       4 = turning bands formulation
%       5 = circuland embedding formulation
% RewriteEPAFlag - Logical - Enables or disables the rewriting of the EPANET system
%       definition file given for input to GIRAFFE. If disabled, then the
%       code will use the original EPANET system definition file as input.
%       *Note*: given EPANET system definition files are not compatible
%       with GIRAFFE, so this flag must be enabled.
% ConvertMaterialFlag - Logical - Enables or disables the conversion of material
%       values from the GIS excel file to values compatible with GIRAFFE. 
%       *Note*: Current formulation requires that this flag be enabled, but
%       this may be made more flexible in the future.
% MedFlag - Logical - This indicates whether to output median values instead
%       of simulating correlated ground motion values
%       0 = do not output median values--simulate correlated ground motion
%           values
%       1 = output median values
% RRLimit - Scalar - Describes the maximum repare rate for any pipe when
%       calculating repair rate for GIRAFFE calculations. This field is
%       more for experimentation on the effect of repair rate, as if this
%       value is set too low, the repair rates will not reflect those
%       resulting from the simulated ground motion.

% **Random Field Simulation Options**
% Spectral_scale - Scalar - Describes the multiplicative factor for the
%       field size of the scaled spectral formulation. Set this value equal
%       to 1 if no scaling is used. Reasonable factors range from 2 - 10.
%       Larger scaling factors typically result in more accurate results
%       with the trade off of more computation time. Values must be a
%       positive integer greater than or equal to 1.
% Sequential_useCoarse - Logical - Enable or disable the use of a preliminary coarse
%       site grid for sequential calculation. Typically would increase the
%       accuracy. 0 = disable, 1 = enable.
% Sequential_coarseSize - Scalar - Describes the size of the coarse grid
%       used. This value corresponds to the number of sites per row and per
%       column of a rectangular site grid.
% Sequential_maxDist - Scalar - Describes the maximum distance from the
%       current site to a previously simulated site to qualify for
%       conditioning, due to screening effect.
% Sequential_maxScreen - Scalar - Describes the maximum number of
%       previously simulated sites that qualify for conditioning, due to
%       screening effect.
% TurningBands_M - Scalar - This indicates the number of harmonics to be
%       used for line simulation for the spectral variant of the turning
%       bands method. Currently unused.
% TurningBands_nLines - Scalar - This indicates the number of lines to be
%       used in the turning bands method. 
% TurningBands_dels - Scalar - This indicates the spacing of bands on each
%       line in the turning bands method.

% **Ground Motion Parameters**
% The lists corresponding to these parameters are found in the next section

% Define_ERF - Scalar - Index of an ERF from list; refer to ERFList for options
% BGSeis - Scalar - Index of an BGSeis from list; refer to BGSeisList for options
% FloatingRuptureOffset - Scalar - floating rupture offset (recommended value is 5)
% AttenuationChoices - Vector - Vector of indices corresponding to
%       attenuation functions from list; refer to AttenuationList for options
% IntensityChoices - Vector - Vector of indices corresponding to intensity
%       measures for list; refer to IntensityList for options
% SAPeriod - Cell - Row or column cell containing spectral periods (in 
%       seconds) for spectral acceleration values. Entries must be positive 
%       numbers expressed as a string. For example, SAPeriod = {1.0 0.2} 
%       will obtain values for the spectral acceleration of periods 1.0 and 
%       0.2 seconds.
% LongBoundLeft - Scalar - west longitudinal boundary for site grid in
%       decimal degrees
% LongBoundRight - Scalar - east longitudinal boundary for site grid in
%       decimal degrees
% LatBoundBot - Scalar - south latitudinal boundary for site grid in
%       decimal degrees
% LatBoundTop - Scalar - north latitudinal boundary for site grid in
%       decimal degrees
% GridSpacing - Scalar - grid spacing in latitude and longitude directions
%       (in meters)
% nSimBatches - Scalar - number of batches of simulations.
% ConstVs30  Scalar  user specified constant Vs30 value. Must be a positive number.
% nGIRAFFESim - Scalar - number of GIRAFFE simulations per ground motion
%       realization
% CaseNum - Vector - indices corresponding to earthquake scenarios to
%       obtain ground motion values and run GIRAFFE. Refer to the metadata
%       file for index values

function [Files,Input,Options, errFlag] = Assemble_Inputs

%% USER DEFINED INPUTS

% **File Organization**

% Identifier = 'MultFull_020215';
% Identifier = 'NoDamage_NoDemand';
% Identifier = 'TestRun2';
Identifier = 'Mult_092416';

siteListFilename = 'SiteList_500m_AGSVs30.txt'; 
% siteListFilename = ''; % Leave blank for program to generate site list
EPAFilename = 'AWSS_Existing_SA79.inp';
% EPAFilename = 'Net1.inp';
GISFilename = 'GIS_TestRun.xls';
GISSheetname = 'GIS_Info';

% **Flags/Options**
OverwriteAllFlag = 0;
OverwriteOpenSHAFlag = 0;
OverwriteGMSimsFlag = 1;
SuppressMessagesFlag = 0;
DefineVs30Flag = 1;
DefineConstVs30Flag = 0;
RandomFieldFlag = 1; % 1 = Covariance, 2 = Spectral, 3 = Sequential, 4 = Turning Bands, 5 = Circulant Embedding
RewriteEPAFlag = 1;
ConvertMaterialFlag = 1;
MedFlag = 0;
RRLimit = 500;
randomSeedFlag = 1;

% **Random Field Simulation Options**
GMseed = 123; % random seed for random field simulation
% Spectral Formulation
Spectral_scale = 1;

% Sequential Formulation
Sequential_useCoarse = 1;
Sequential_coarseSize = 5;
Sequential_maxDist = 30;
Sequential_maxScreen = 30;

% Turning Bands Formulation
TurningBands_M = 100;
TurningBands_nLines = 16;
TurningBands_dels = 0.5;

% **Ground motion parameters**

% The lists corresponding to these parameters are found in the next section
% Choose ERF from list; refer to ERFList for options
Define_ERF = 1; 
% Choose BGSeis from list; refer to BGSeisList for options
BGSeis = 2; 
% Choose floating rupture offset (recommended value is 5)
FloatingRuptureOffset = 5; 
% Choose attenuation functions from list; refer to AttenuationList for options
AttenuationChoices = [2];
% Choose intensity measure from list; refer to IntensityList for options
IntensityChoices = [2]; 
% Choose SA periods (seconds)
SAPeriod = {'1.0' '0.2'}; 
% Choose boundaries for site grid (decimal degrees)

% San Francisco
% LongBoundLeft = -122.514; 
% LongBoundRight = -122.3608; 
% LatBoundBot = 37.7065; 
% LatBoundTop = 37.8121;

% Shelby County
% LongBoundLeft = -90.176; 
% LongBoundRight = -89.637; 
% LatBoundBot = 35.001; 
% LatBoundTop = 35.413;

% Extended San Francisco
LongBoundLeft = -122.514; 
LongBoundRight = -121.96; 
LatBoundBot = 37.538; 
LatBoundTop = 37.950;


% Choose grid spacing (meters) in latitude and longitude directions
GridSpacing = 2000;
% Number of batches of simulations 
nSimBatches = 50;
% Specify constant vs30 instead
ConstVs30 = 760;
% Number of GIRAFFE simulations per ground motion map
nGIRAFFESim = 3;
% Random seed for GIRAFFE simulations
GIRAFFEseed = 10;

% Chosen earthquake scenarios, indices from the metadata list
% CaseNum = [60]; % 6.9 Hayward
% CaseNum = [282]; % NSA 6.5
% CaseNum = [134]; % NSA 7.9
% CaseNum = [156]; % NSA 7.2
% CaseNum = [60 156 282];
% CaseNum = [6 60 134 1534 1877];
% CaseNum = [6 60 1534 1877];
% CaseNum = [1534];
% CaseNum = [61 72 93 98 110 111 112 113 114 123 124 125 128 130 132 133 136 137 138 139 140 256 269];
CaseNum = 1:2430;
% CaseNum = 1:5;
% CaseNum = [61 62 63 64 84 91 93 95 109 110 111 112 113 116 117 118 119 120 128 129 130 131 132 133 135 136 137 138 139];
% CaseNum = [61 62 63 109 110 113 117 120 125 132 135 136 137 138 139 243 264 269 270 299 300 327 348 383 389 408 420 421 459 1547 1555 1571 1573 1574 1575];
% CaseNum = [61 62 63 109 110 113 117 120 125 132 135 136 137 138 139 269 270 299 300 327 348 383 389 408 420 421 459 1472 1555 1571 1573 1574 1575];
%% Lists
% These lists indicate the options allowed for the corresponding inputs to
% OpenSha. The  number before each line indicates the number the 
% user must assign to the corresponding input variables to choose that
% option.

% ERFList = {
%   1 'WGCEP (2007) UCERF2 - Single Branch';
%   2 'USGS/CGS 2002 Adj. Cal. ERF';
%   3 'WGCEP UCERF 1.0 (2005)'; 
%   4 'GEM1 CEUS ERF'};
%        
% BGSeisList = {
%   1 'Include';
%   2 'Exclude';
%   3 'Only Background'};
%           
% AttenuationList = {
%   1 'Campbell & Bozorgnia (2008)';
%   2 'Boore & Atkinson (2008)';
%   3 'Abrahamson & Silva (2008)';
%   4 'Chiou & Youngs (2008)';
%   5 'Boore & Atkinson (2006)';
%   6 'Chiou & Youngs (2006)';
%   7 'Campbell & Bozorgnia (2006)';
%   8 'Boore, Joyner & Fumal (1997)';
%   9 'Field (2000)';
%   10 'ShakeMap (2003)';
% 
% IntensityList = {
%   1 'PGA';
%   2 'PGV';
%   3 'SA';

%% Error checking
Check_Input_Errors
if errFlag == 1
    return
end

%% Additional Inputs (Default Values)
% Finds the path to the folder containing the entire package
% (AnalysisPackage)
BaseFolder = findBaseFolder;
UserInputFolder = [BaseFolder 'UserInput\'];
MasterFolder = [BaseFolder 'MasterFolders\MasterFolder_' Identifier '\'];
SimulationFolder = [MasterFolder 'Simulations\'];
OpenSHAFolder = [BaseFolder 'OpenSHA\'];
OpenSHAInputFilename = ['OpenSHAInput_' Identifier '.txt'];
OpenSHAOutputFolderName = ['Output_' Identifier '\'];
siteListFilepath = [UserInputFolder siteListFilename];

EPAFilePath = [UserInputFolder EPAFilename];
EPAInfoFilename = 'EPANETInfo.mat';
EPAInfoFilepath = [MasterFolder EPAInfoFilename];
EPARewriteFilename = 'EPANETRewrite.txt';
EPARewriteFilepath = [MasterFolder EPARewriteFilename];
EPASupplementFilepath = [findContainingFolder 'EPANETSysDefSupplement.txt'];

GISFilePath = [UserInputFolder GISFilename];

GMOIFilename = 'GroundMotionsOfInterest.mat';
GMOIFilepath = [MasterFolder GMOIFilename];
LinkedNetworkFilename = 'LinkedNetwork.mat';
LinkedNetworkFilepath = [MasterFolder LinkedNetworkFilename];
AssignedGMFilename = 'AssignedGroundMotions.mat';
AssignedGMFilepath = [MasterFolder AssignedGMFilename];
RepairRateFilename = 'RepairRate.mat';
RepairRateFilepath = [MasterFolder RepairRateFilename];
GiraffeRRBasename = 'GIRAFFE_RR_';
GiraffeRRFolder = [MasterFolder 'GIRAFFE_RR\'];
GiraffeOutFolder = [MasterFolder 'GIRAFFE_Out\'];

GiraffeExecPath = [BaseFolder 'GiraffeCode\Release\'];

%% Basic Input Structure Creation
Files.Identifier = Identifier;
Files.MasterFolder = MasterFolder;
Files.SimulationFolder = SimulationFolder;
Files.OpenSHAFolder = OpenSHAFolder;
Files.OpenSHAInputFilename = OpenSHAInputFilename;
Files.OpenSHAOutputFolderName = OpenSHAOutputFolderName;
Files.siteListFilepath = siteListFilepath;
Files.EPAFilePath = EPAFilePath;
Files.EPAInfoFilepath = EPAInfoFilepath;
Files.EPARewriteFilepath = EPARewriteFilepath;
Files.EPASupplementFilepath = EPASupplementFilepath;
Files.GISFilePath = GISFilePath;
Files.GISSheetname = GISSheetname;
Files.GMOIFilepath = GMOIFilepath;
Files.LinkedNetworkFilepath = LinkedNetworkFilepath;
Files.AssignedGMFilepath = AssignedGMFilepath;
Files.RepairRateFilepath = RepairRateFilepath;
Files.GiraffeRRBasename = GiraffeRRBasename;
Files.GiraffeRRFolder = GiraffeRRFolder;
Files.GiraffeOutFolder = GiraffeOutFolder;
Files.GiraffeExecPath = GiraffeExecPath;

Input.Define_ERF = Define_ERF; 
Input.BGSeis = BGSeis; 
Input.FloatingRuptureOffset = FloatingRuptureOffset; 
Input.AttenuationChoices = AttenuationChoices;
Input.IntensityChoices = IntensityChoices; 
Input.SAPeriod = SAPeriod; 
Input.CaseNum = CaseNum;

GridSpacingLat = GridSpacing;
GridSpacingLong = GridSpacing; 
Gridspecs = [LongBoundLeft LongBoundRight LatBoundBot LatBoundTop GridSpacingLat GridSpacingLong];
Input.Gridspecs = Gridspecs;
Input.nSimBatches = nSimBatches;
Input.ConstVs30 = ConstVs30;
Input.nGIRAFFESim = nGIRAFFESim;
Input.GIRAFFEseed = GIRAFFEseed;
Input.GMseed = GMseed;

SpectralParams.scale = Spectral_scale;
SpectralParams.spacing = GridSpacing/1000; % Spectral spacing in km instead of m
Input.SpectralParams = SpectralParams;

SequentialParams.useCoarse = Sequential_useCoarse;
SequentialParams.coarseSize = Sequential_coarseSize;
SequentialParams.maxDist = Sequential_maxDist;
SequentialParams.maxScreen = Sequential_maxScreen;
Input.SequentialParams = SequentialParams;

TurningBandsParams.M = TurningBands_M;
TurningBandsParams.nLines = TurningBands_nLines;
TurningBandsParams.dels = TurningBands_dels;
Input.TurningBandsParams = TurningBandsParams;

CircEmbParams.spacing = GridSpacing/1000;
Input.CircEmbParams = CircEmbParams;

% OverwriteFlags, index/effect
% 1 / Overwrite OpenSHA inputs and outputs
% 2 / Overwrite Ground motion simulation realizations
OverwriteFlags = zeros(2,1);
if OverwriteAllFlag == 1
    OverwriteFlags = OverwriteFlags + 1;
elseif OverwriteOpenSHAFlag == 1
    OverwriteFlags = OverwriteFlags + 1;
elseif OverwriteGMSimsFlag == 1
    OverwriteFlags(2:end) = OverwriteFlags(2:end) + 1;
end

Options.OverwriteFlags = OverwriteFlags;
Options.OverwriteFlag = 1; % temp
Options.SuppressMessagesFlag = SuppressMessagesFlag;
Options.DefineVs30Flag = DefineVs30Flag;
Options.DefineConstVs30Flag = DefineConstVs30Flag;
Options.RandomFieldFlag = RandomFieldFlag;
Options.RewriteEPAFlag = RewriteEPAFlag;
Options.ConvertMaterialFlag = ConvertMaterialFlag;
Options.MedFlag = MedFlag;
Options.RRLimit = RRLimit;
Options.randomSeedFlag = randomSeedFlag;

if ~exist(MasterFolder,'dir')
    mkdir(MasterFolder);
end
save([MasterFolder 'InputSpecs.mat'],'Files','Input','Options')

end

function BaseFolder = findBaseFolder

ContainingFolder = findContainingFolder;
ContainingFolder = ContainingFolder(1:end-1);
inds = find(ContainingFolder == '\');
BaseFolder = ContainingFolder(1:inds(end));

end

function ContainingFolder = findContainingFolder

fullpath = mfilename('fullpath');
codename = mfilename;
ContainingFolder = fullpath(1:end - length(codename));

end