% Covariance_Sim.m
% Jason Wu
% 1/26/15
% This obtains a realization of the correlated epsilons for each site
% and each earthquake scenario case using the covariance formulation
% 
% The model variogram used can be found in:
% Jayaram, N. Probabilistic Seismic Lifeline Risk Assessment Using 
% Efficient Sampling and Data Reduction Techniques. Ph. D. Dissertation, 
% Stanford University: California, 2010. http://purl.stanford.edu/hp846fv1725
% 
% INPUTS (if input structure is not used):
%
% MasterFolder : string : path to the folder that will contain relavent
%       precomputations for these calculations
% numScenarios : Scalar : Scalar defining the number of realizations
%       of epsilons corresponding to the number of earthquake scenarios
%       simulated in OpenSHA
% siteList : Matrix : matrix that defines the latitude/longitude
%       coordinates of the sites of interest at which we will be simulating
%       ground motion values
% period: Scalar : Scalar indicating the period of the spectral
%       acceleration for which epsilons are being calculated. This
%       value should correspond with the following:
%       period = 1 if the intensity measure is PGV
%       period = 0 if the intensity measure is PGA
%       period = # if the intensity measure is SA, where # represents
%           the period corresponding to the SA
%
% OUTPUTS:
%
% epsilons : Matrix : numSimulations x n matrix indicating the
%       realizations of the correlated epsilons for each n sites for
%       each earthquake scenario (numSimulations of them)


function epsilons = Covariance_Sim(SimInput)

if nargin == 0
    Identifier = '030414_SF_05km_large';
    BaseFolder = 'C:\Jason Files\Baker Research 13\';
    MasterFolder = [BaseFolder 'MasterFolders\MasterFolder_' Identifier '\'];
    
    OverwriteFlag = 1;
    
    period = 0;
    siteList = [0 0; ...
                0 0.5; ...
                0.5 0; ...
                0.5 0.5];
    numScenarios = 500;
    
else
    MasterFolder = SimInput.MasterFolder;
    
    OverwriteFlag = SimInput.OverwriteFlag;
    
    period = SimInput.period;
    siteList = SimInput.siteList;
    numScenarios = SimInput.numScenarios;
    
end

paramFilepath = [MasterFolder 'Params_Covariance.mat'];
 
%% Calculate or load the required parameters for the random field simulation
if ~exist(paramFilepath,'file') || OverwriteFlag == 1
    
    nSites = size(siteList,1);
    h = getDistanceMatrix(siteList,nSites);
    
    % Perform calculation
    
    if period < 1
        b = 40.7 - 15*period; % For short periods, clustering in Vs30
        %     b = 8.5 + 17.2*period; % For short periods, no clustering in Vs30
    else
        b = 22.0 + 3.7*period; % For long periods, period >= 1
    end
    
    SIGMA = exp(-3*h/ b);
    chol_sigma = chol(SIGMA);
    save(paramFilepath,'chol_sigma')

else
   load(paramFilepath);
end

epsilons = randn(numScenarios,size(chol_sigma,1))*chol_sigma;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = getDistanceMatrix(siteList,nSites)

h = zeros(nSites,nSites);

% Calculate the upper triangle of the distance matrix, as the distance
% matrix is symmetric and has zeros on the diagonal.
for siteI = 1:nSites
    for siteJ = (siteI+1):nSites
        LatLong1 = siteList(siteI,1:2);
        LatLong2 = siteList(siteJ,1:2);
        h(siteI,siteJ) = distanceFromLongLat(LatLong1,LatLong2);
    end
end

% Add in the lower triangle to form the whole distance matrix
h = h + h';

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function distance = distanceFromLongLat(LatLong1,LatLong2)
% This calculates the distance in kilometers given the lat/long
% coordiates at two sites.
%
% INPUTS:
%
% LatLong1 : Vector : 2 x 1 vector indicating the latitude/longitude
%       coordinates for the first site (decimal degrees)
% LatLong2 : Vector : 2 x 1 vector indicating the latitude/longiude
%       coordinates for the second site (decimal degrees)
%
% OUTPUTS:
%
% distance : Scalar : Scalar indicating the distance between the two
%       latitude/longitude coordinates in kilometers
%
% NOTE: We are currently using the Haversine formula to calculate

radius = 6371; % Radius of the earth in km

LatLong1 = LatLong1 * pi/180; % change degrees to radians
LatLong2 = LatLong2 * pi/180; % change degrees to radians

dLatLong = LatLong2 - LatLong1;
dlat = dLatLong(1);
dlong = dLatLong(2);
lat1 = LatLong1(1);
lat2 = LatLong2(1);

% Change of angle using Haversine formula
dAngle = 2*asin(sqrt(sin(dlat/2)^2+cos(lat1)*cos(lat2)*sin(dlong/2)^2));

% Change of angle using spherical law of cosines
%     dAngle = acos(sin(lat1)*sin(lat2)+cos(lat1)*cos(lat2)*cos(dlong));

% Calculate the separation distance between the two points
distance = radius*dAngle;

end


