% Run_Precalculations.m
% Jason Wu
% 5/15/14
%
% This function serves as a wrapper function to generate the relevant
% OpenSHA metadata for the area of interest and a site list prior to full
% simulation.

function Run_Precalculations

[Files,Input,Options,errFlag] = Assemble_Inputs;
if errFlag == 1
    return
end

GenerateMetadata(Files,Input,Options);

Site_Creator(Files, Input, Options);


end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Function: GenerateMetadata
% This function generates the OpenSHA metadata for the area of interest by
% calling the OpenSHA event set generator with the bounds of the site grid
% as the sites of interest.
function GenerateMetadata(Files,Input,Options)

% Preparation
MasterFolder = Files.MasterFolder;
OpenSHAFolder = Files.OpenSHAFolder;
OpenSHAInputFilename = Files.OpenSHAInputFilename;
OpenSHAOutputFolderName = Files.OpenSHAOutputFolderName;

if ~exist(MasterFolder,'dir')
    mkdir(MasterFolder)
end

if Options.SuppressMessagesFlag == 0
    disp('Generating Metadata...')
end

% Adjust input structures for precalculation purposes
Options.OverwriteFlag = 1;
Options.SuppressMessagesFlag = 1;
gridspecs = Input.Gridspecs;
gridspecs(5) = (gridspecs(4) - gridspecs(3))*111011.64;
gridspecs(6) = (gridspecs(2) - gridspecs(1))*89011.64;
Input.Gridspecs = gridspecs;

% Create the site list for precalculation purposes, which only defines the
% 4 corners of the defined site grid
Site_Creator(Files, Input, Options);

% Create the OpenSHA using the precalculation site list to obtain the
% earthquake scenarios relevant to this area
Opensha_Input_Writer(Files,Input,Options);

% Call OpenSHA event set generator
Filename = OpenSHAInputFilename;
OutputFolder = OpenSHAOutputFolderName;
OpenSHAOutputFolder = [OpenSHAFolder OpenSHAOutputFolderName];

origPath = cd;
cd(Files.OpenSHAFolder);
dos(['java -jar -Xmx500M IM_EventSetCalc_v3_0_ASCII.jar ' Filename ' ' OutputFolder]);
delete(Filename)
cd(origPath);


% Obtain the meta data
metaDataFilename = 'src_rup_metadata.txt';
metaDataFilepath = [OpenSHAOutputFolder metaDataFilename];
[metaData, metaDataDescription, ~] = extractMetaData(metaDataFilepath);

% Data Saving/Writing
% Save the meta data information
metaDataAll = [num2cell(1:size(metaData,1))' num2cell(metaData) metaDataDescription];
metaDataHeader = {'Index' 'SourceID' 'RuptureID' 'Annualized Rate' 'Magnitude' 'Name'};
metaDataAll = [metaDataHeader; metaDataAll];
metaDataPath = [MasterFolder 'metadata.mat'];
save(metaDataPath,'metaDataAll');


% Also write the metadata to an excel file for easier reading
excelFilename = ['Metadata.xls'];
excelpath = [MasterFolder excelFilename];
warning('off','MATLAB:xlswrite:AddSheet');
xlswrite(excelpath,metaDataAll,'metadata');

rmdir(OpenSHAOutputFolder,'s')

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Function: extractMetaData
% This function extracts the metadata from OpenSHA output and produces a
% text file and an excel file that describes the relevant earthquake
% scenarios in this area.
function [metaData, metaDataDescription, numScenarios] = extractMetaData(metaDataFilepath)

fid = fopen(metaDataFilepath,'r');
metaDataCell = textscan(fid,'%d %d %f %f %s %s %s %s %s %s %s %s %s');
fclose(fid);

sourceIDkey = double(metaDataCell{1}); 
ruptureIDkey = double(metaDataCell{2}); 
annualizedRate = metaDataCell{3};
mag = metaDataCell{4};
metaData = [sourceIDkey ruptureIDkey annualizedRate mag]; % Numerical component of the meta data

% Name of source/rupture case. Note that due to white space in the name 
% inside the text file, the name is separated into different column cells.
metaDataDescription = [];
for i = 5:13
   metaDataDescription = [metaDataDescription metaDataCell{i}];
end

% Collapse the name into a single cell for each source/rupture case,
% creating a single column cell
for i = 1:length(metaDataDescription)
    for j = 2:size(metaDataDescription,2)
        metaDataDescription{i,1} = [metaDataDescription{i,1} metaDataDescription{i,j}];
    end
end

metaDataDescription = metaDataDescription(:,1);
numScenarios = size(metaData,1); % Number of earthquake scenarios

end
