% SiteCreator.m
% Jason Wu 
% 1/26/15

% edited from arcGIStoOpenSHAExperimentGenerator.m by Mahalia Miller
%
% This function creates the input coordinate grid and V_s30 for openSHA
%
% INPUTS:
% 
% **If inputs are hard coded (i.e. not called by other functions):
% LongBoundLeft - Scalar - west longitudinal boundary for site grid in
%       decimal degrees
% LongBoundRight - Scalar - east longitudinal boundary for site grid in
%       decimal degrees
% LatBoundBot - Scalar - south latitudinal boundary for site grid in
%       decimal degrees
% LatBoundTop - Scalar - north latitudinal boundary for site grid in
%       decimal degrees
% GridSpacing - Scalar - grid spacing in latitude and longitude directions
%       (in meters)
% filepath : String : full path/directory and name of the file to which the
%       site list generated is saved.
% 
% **If inputs are from arguments (i.e. called by other functions): 
% gridspecs: Vector : Vector containing the specifications to generate the
%       site list. This should be in the following format:
%       [LongBoundLeft; LongBoundRight; LatBoundBot; LatBoundTop; ...
%       GridSpacingLat; GridSpacingLong]
% filepath : String : full path/directory and name of the file to which the
%       site list generated is saved. This is equivalaent to the following:
%       [folderloc filename]

function Site_Creator(Files, Input, Options)

%% Inputs
if nargin == 0

%   Site Grid Specs
    LongBoundLeft = -122.5;
    LongBoundRight = -122.36;
    LatBoundBot = 37.7;
    LatBoundTop = 37.81;

    GridSpacingLat = 300;
    GridSpacingLong = 300;

%   File location
    Identifier = 'Example';
    BaseFolder = 'C:\Jason Files\Baker Research 13\';
    MasterFolder = [BaseFolder 'Masterfolder_' Identifier '\'];
    
    filename = 'SiteList.txt';

    filepath = [MasterFolder filename];
    
    ConstVs30 = 760;
    OverwriteFlag = 1;
    suppressMessagesFlag = 0;
    DefineConstVs30Flag = 1;

else
    filepath = Files.siteListFilepath;
    MasterFolder = Files.MasterFolder;
    
    gridspecs = Input.Gridspecs;
    LongBoundLeft = gridspecs(1); 
    LongBoundRight = gridspecs(2); 
    LatBoundBot = gridspecs(3); 
    LatBoundTop = gridspecs(4); 
    GridSpacing = gridspecs(5);
%     GridSpacingLat = gridspecs(5);
%     GridSpacingLong = gridspecs(6);
    ConstVs30 = Input.ConstVs30;
    
    OverwriteFlag = Options.OverwriteFlags(1);
    suppressMessagesFlag = Options.SuppressMessagesFlag;
    DefineConstVs30Flag = Options.DefineConstVs30Flag;
end

%% Preparation
GridSpacingLat = GridSpacing;
GridSpacingLong = GridSpacing;

localFilepath = [MasterFolder 'SiteList.txt'];

if ~exist(MasterFolder,'dir')
    mkdir(MasterFolder);
end

%% Calculations

% If the local site list file exists and we do not want to overwrite it,
% skip new calculations
if exist(localFilepath,'file') == 2 && OverwriteFlag == 0
    return
end

% If the specified file path for a site list exists, create a local copy of
% this site list for future use
if exist(filepath, 'file') == 2
    success = copyfile(filepath, localFilepath);
    if success ~= 0 || isequal(filepath,localFilepath)
        return
    else
        if suppressMessagesFlag == 0
            fprintf('Cannot find %s, creating new site list...\n', filepath)
        end
    end
else
    if suppressMessagesFlag == 0
        fprintf('Creating new site list...\n')
    end
end

% Otherwise, generate a new site list

% Rough estimate of meters/decimal degrees latitude
m2ddLat = 111111; 
% Rough estimate of meters/decimal degrees longitude
m2ddLong = cosd(mean([LatBoundTop LatBoundBot]))*111111; 

GridSpacingLat = GridSpacingLat/m2ddLat; % Change from meters to degrees
GridSpacingLong = GridSpacingLong/m2ddLong; % Change from meters to degrees

Xs = LongBoundLeft:GridSpacingLong:LongBoundRight; % Set of X coordinates for equally
                                                   % spaced sites
Ys = LatBoundBot:GridSpacingLat:LatBoundTop; % Set of Y coordinates for equally 
                                             % spaced sites

nSitesX = length(Xs);
nSitesY = length(Ys);

save([MasterFolder 'siteInfo.mat'],'nSitesX','nSitesY','GridSpacing')
                                             
[XI,YI] = meshgrid(Xs,Ys);

xx = reshape(XI,size(XI,1)*size(XI,2), 1);% makes a column
yy = reshape(YI,size(YI,1)*size(YI,2), 1);% makes a column
lon = xx;
lat = yy;

% Write into text file
% open the file with write permission
fid = fopen(localFilepath, 'w+');

if DefineConstVs30Flag == 1;
    fprintf(fid, '%6.4f %6.4f %6.4f\n', [lat lon ConstVs30*ones(size(lat))]');
else
    fprintf(fid, '%6.4f %6.4f\r\n', [lat lon]');
end
fclose(fid);